@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -452,10 +452,23 @@ ShapeAlgorithms.autoSplit = function(shape, otherShapes, position, extend) {
         cutPos2 = undefined;
     }
 
-    return ShapeAlgorithms.autoSplitManual(shape, cutDist1, cutDist2, cutPos1, cutPos2, position);
+    return ShapeAlgorithms.autoSplitManual(shape, cutDist1, cutDist2, cutPos1, cutPos2, position, extend);
 };
 
-ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, cutPos2, position) {
+/**
+ * Cut shape at given distances / positions.
+ *
+ * \return Array of three new shapes which each might be undefined if its
+ * length would otherwise be 0.
+ * The first shape is the rest at the start of the shape.
+ * The second shape is the rest at the end of the shape.
+ * The third shape is the segment self in its new shape.
+ */
+ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, cutPos2, position, extend) {
+    if (isNull(extend)) {
+        extend = false;
+    }
+
 //    if (!isCircleShape(shape) && !isFullEllipseShape(shape) &&
 //        !isXLineShape(shape) && !isRayShape(shape)) {
 
@@ -624,31 +637,33 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
         rest2.trimStartPoint(cutDist2);
 
         segment = shape.clone();
-        var l1 = segment.getLength();
-        segment.trimStartPoint(cutDist1);
-        var l2 = segment.getLength();
-        segment.trimEndPoint(cutDist2 - (l1-l2));
+        //var l1 = segment.getLength();
+        segment.setStartAngle(segment.getCenter().getAngleTo(cutPos1));
+        //segment.trimStartPoint(cutDist1);
+        //var l2 = segment.getLength();
+        //segment.trimEndPoint(cutDist2 - (l1-l2));
+        segment.setEndAngle(segment.getCenter().getAngleTo(cutPos2));
 
-        var angleLength1 = rest1.getAngleLength(true);
-        var angleLength2 = rest2.getAngleLength(true);
+        if (!extend) {
+            var angleLength1 = rest1.getAngleLength(true);
+            var angleLength2 = rest2.getAngleLength(true);
+            if (angleLength1+angleLength2 > shape.getAngleLength()) {
+                rest1.trimEndPoint(cutDist2);
+                rest2.trimStartPoint(cutDist1);
 
-        if (angleLength1+angleLength2 > shape.getAngleLength()) {
-            rest1.trimEndPoint(cutDist2);
-            rest2.trimStartPoint(cutDist1);
+                segment.trimStartPoint(cutDist2);
+                segment.trimEndPoint(cutDist1);
 
-            segment.trimStartPoint(cutDist2);
-            segment.trimEndPoint(cutDist1);
+                angleLength1 = rest1.getAngleLength(true);
+                angleLength2 = rest2.getAngleLength(true);
+            }
+            if (angleLength1<1.0e-5) {
+                rest1 = undefined;
+            }
 
-            angleLength1 = rest1.getAngleLength(true);
-            angleLength2 = rest2.getAngleLength(true);
-        }
-
-        if (angleLength1<1.0e-5) {
-            rest1 = undefined;
-        }
-
-        if (angleLength2<1.0e-5) {
-            rest2 = undefined;
+            if (angleLength2<1.0e-5) {
+                rest2 = undefined;
+            }
         }
     }
 
@@ -659,8 +674,8 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
             rest2 = undefined;
         }
         else {
-            var angle1 = shape.getCenter().getAngleTo(cutDist1);
-            var angle2 = shape.getCenter().getAngleTo(cutDist2);
+            var angle1 = shape.getCenter().getAngleTo(cutPos1);
+            var angle2 = shape.getCenter().getAngleTo(cutPos2);
 
             rest1 = new RArc(
                         shape.getCenter(),
@@ -952,7 +967,8 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
 
 /**
  * \return The two distances along the given shape identifying the
- * intersections points closest to the given position.
+ * intersections points closest to the given position along with the cut positions:
+ * [ [cutDist1, cutDist2], [cutPos1, cutPos2] ]
  *
  * \param onShape True: only return intersections on the shape
  * (for trimming, breaking, default).
@@ -1022,6 +1038,10 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
     if (isPolylineShape(shape) && shape.isGeometricallyClosed()) {
         circular = true;
     }
+    if (isArcShape(shape)) {
+        circular = true;
+    }
+
 //    if (isSplineShape(shape) && shape.isGeometricallyClosed()) {
 //        circular = true;
 //    }
@@ -1046,7 +1066,7 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
     var cutPos1 = undefined;
     var cutPos2 = undefined;
 
-    // for circular shapes, also find intersections closest to start point:
+    // for circular shapes, also find intersections closest to and furthest from start point:
     var cutDistMax = undefined;
     var cutDistMin = undefined;
     var cutPosMax = undefined;
@@ -1057,11 +1077,11 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
 
         if (isEllipseShape(shape)) {
             dist = RMath.getAngleDifference(orthoLine.getAngle(), shape.getCenter().getAngleTo(ip));
-            if (isNull(cutDist1) || dist>cutDist1) {
+            if (isNull(cutDist1) || dist<cutDist1) {
                 cutPos1 = ip;
                 cutDist1 = dist;
             }
-            if (isNull(cutDist2) || dist<cutDist2) {
+            if (isNull(cutDist2) || dist>cutDist2) {
                 cutPos2 = ip;
                 cutDist2 = dist;
             }
@@ -1097,7 +1117,7 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
                 }
 
                 if (circular) {
-                    if (isNull(cutDistMin) || dist<cutDistMin) {
+                    if (isNull(cutDistMin) || (dist<cutDistMin && (dist>pDist || isPolylineShape(shape)))) {
                         cutDistMin = dist;
                         cutPosMin = ip;
                     }
@@ -1106,6 +1126,27 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
         }
     }
 
+    if (circular) {
+        if (isNull(cutDist1)) {
+            cutDist1 = cutDistMax;
+            cutPos1 = cutPosMax;
+        }
+        if (isNull(cutDist2)) {
+            cutDist2 = cutDistMin;
+            cutPos2 = cutPosMin;
+        }
+    }
+
+    if (isEllipseShape(shape) && shape.isReversed()) {
+        var dummy = cutPos1;
+        cutPos1 = cutPos2;
+        cutPos2 = dummy;
+
+        dummy = cutDist1;
+        cutDist1 = cutDist2;
+        cutDist2 = dummy;
+    }
+
     // open shape: cut to start or end point:
     if (!isCircleShape(shape) &&
         !isFullEllipseShape(shape) &&
@@ -1128,17 +1169,6 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
         shape.reverse();
     }
 
-    if (circular) {
-        if (isNull(cutDist1)) {
-            cutDist1 = cutDistMax;
-            cutPos1 = cutPosMax;
-        }
-        if (isNull(cutDist2)) {
-            cutDist2 = cutDistMin;
-            cutPos2 = cutPosMin;
-        }
-    }
-
     return [ [cutDist1, cutDist2], [cutPos1, cutPos2] ];
 
     /*
@@ -1803,9 +1833,9 @@ ShapeAlgorithms.splineToLineOrArc = function(spline, tolerance, linesOnly) {
                 }
             }
 
-            if (splineIsArc && arc.getRadius()>1000.0) {
-                return new RLine(startPoint, endPoint);
-            }
+//            if (splineIsArc && arc.getRadius()>1000.0) {
+//                return new RLine(startPoint, endPoint);
+//            }
 
             if (splineIsArc) {
                 return arc;
@@ -1833,11 +1863,7 @@ ShapeAlgorithms.splineToLineOrArc = function(spline, tolerance, linesOnly) {
  * Converts the given circle into an arc with the given start angle or 0.
  */
 ShapeAlgorithms.circleToArc = function(circle, startAngle) {
-    if (isNull(startAngle)) {
-        startAngle = 0.0;
-    }
-
-    return new RArc(circle.getCenter(), circle.getRadius(), startAngle, startAngle + 2*Math.PI, false);
+    return circle.toArc(startAngle);
 };
 
 /**
